import torchvision.models as models


def Model(pretrained_model):
    model = getattr(models, '%s' % pretrained_model)(pretrained=True)
    # Set the model to Evaluation mode
    model.eval()

    return model



from setuptools import setup, find_packages

VERSION = '0.0.3'
DESCRIPTION = 'Feacher - An Image Feature Vector Extraction Library'
LONG_DESCRIPTION = 'A simple PyTorch based image feature \
                    extractor using Pre-Trained Models \
                    useful for Transfer Learning applications.'

setup(
        name="feacher",
        version=VERSION,
        author="Hari Prasad",
        author_email="h4ri.prasad@gmail.com",
        url="https://github.com/qpochlabs/feacher",
        license="MIT",
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        packages=find_packages(),
        install_requires=[
            'torch',
            'Pillow',
            'torchvision',
        ],
        keywords=['python', 'pytorch', 'torchvison', 'feature-extraction'],
        classifiers=[
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Science/Research",
            "Intended Audience :: Developers",
            "Intended Audience :: Education",
            "Programming Language :: Python :: 2",
            "Programming Language :: Python :: 3",
            "Operating System :: Microsoft :: Windows",
            "Operating System :: MacOS :: MacOS X",
            "Operating System :: POSIX :: Linux",
            "Natural Language :: English",
        ]
)



import torch
from feacher import preprocess as ps
from feacher import model as ml


def feature_vector(pretrained_model, image, resize_dim, layer):
    model = ml.Model(pretrained_model=pretrained_model)
    input_image = ps.image_preprocess(image, resize_dim)
    input_batch = input_image.unsqueeze(0)

    with torch.no_grad():
        feature_vector = None

        def hook(module_, input_, feature_vector_):
            nonlocal feature_vector
            feature_vector = feature_vector_

        if(layer[:4] == 'feat'):
            m = layer[9:][:1]
            a_hook = model.features[int(m)].register_forward_hook(hook)

        if(layer[:4] == 'clas'):
            m = layer[11:][:1]
            a_hook = model.classifier[int(m)].register_forward_hook(hook)

        a_hook = model.avgpool.register_forward_hook(hook)
        model(input_batch)
        a_hook.remove()

        return feature_vector
